package com.huimv.eartag2.service.impl;

import com.huimv.eartag2.common.dao.entity.EartagDeviceRegisterEntity;
import com.huimv.eartag2.common.utils.BizConst;
import com.huimv.eartag2.service.ICacheService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class CacheServiceImpl implements ICacheService {
    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public Object getDeviceCodeByChipId(String chipId) {
        String  hashKey = BizConst.CHIP_DEVICE_PREFIX;
        return redisTemplate.opsForHash().get(hashKey,chipId);
    }

    @Override
    public void setDeviceCodeByChipId(String chipId,String deviceCode) {
        String  hashKey = BizConst.CHIP_DEVICE_PREFIX;
        redisTemplate.opsForHash().put(hashKey,chipId,deviceCode);
    }

    @Override
    public boolean checkEffectiveDeviceSet(String idCode) {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
        return redisTemplate.opsForSet().isMember(setKey,idCode);
    }

    @Override
    public long putEffectiveDeviceSet(String idCode) {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
        return redisTemplate.opsForSet().add(setKey,idCode);
    }

    @Override
    public Object getFarmIdByDeviceCode(String deviceCode) {
        Object farmIdObj = redisTemplate.opsForHash().get(BizConst.DEVICE_REGISTER_PREFIX + deviceCode,"farmId");
        return farmIdObj;
    }
    @Override
    public void putDeviceRegister(String deviceCode, EartagDeviceRegisterEntity deviceRegisterEntity) {
        //--更新redis中注册表数据
        String deviceRegHashKey = BizConst.DEVICE_REGISTER_PREFIX + deviceCode;
        //将Entity转为Map
        Map registerMap = RegisterEntityToMap(deviceRegisterEntity);
        redisTemplate.opsForHash().putAll(deviceRegHashKey, registerMap);
    }
    private Map RegisterEntityToMap(EartagDeviceRegisterEntity deviceRegisterEntity) {
        Map map = new HashMap();
        map.put("Id", deviceRegisterEntity.getId());
        map.put("deviceCode", deviceRegisterEntity.getDeviceCode());
        map.put("farmId", deviceRegisterEntity.getFarmId());
        map.put("farmName", deviceRegisterEntity.getFarmName());
        map.put("typeF", deviceRegisterEntity.getTypeF());
        map.put("stage", deviceRegisterEntity.getStage());
        map.put("pigpenId", deviceRegisterEntity.getPigpenId());
        map.put("unitId", deviceRegisterEntity.getUnitId());
        map.put("registerTime", deviceRegisterEntity.getRegisterTime());
        map.put("lastTime", deviceRegisterEntity.getLastTime());
        map.put("activeStatus", deviceRegisterEntity.getActiveStatus());
        map.put("activeTime", deviceRegisterEntity.getActiveTime());
        map.put("deviceStatus", deviceRegisterEntity.getDeviceStatus());
        map.put("deviceAlias", deviceRegisterEntity.getDeviceAlias());
        map.put("location", deviceRegisterEntity.getLocation());
        map.put("remark", deviceRegisterEntity.getRemark());
        return map;
    }

}
