package com.huimv.eartag2.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.eartag2.common.dao.entity.EartagDataEntity;
import com.huimv.eartag2.common.dao.repo.EartagDataRepo;
import com.huimv.eartag2.mapper.EartagData2Mapper;
import com.huimv.eartag2.pojo.EartagData;
import com.huimv.eartag2.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class EartagServiceImpl implements IEartagService {
    @Autowired
    RabbitTemplate rabbitTemplate;  //使用RabbitTemplate,这提供了接收/发送等等方法
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private EartagDataRepo eartagDataRepo;

    //处理耳标信息
    @Override
    public void handleEartagInfo() {
        //--处理注册表数据
        handleDeviceRegister();
        //--处理在线记录数据
        handleDeviceOnline();
        //--处理状态数据
        handleDeviceAllState();
    }

    //--处理状态数据
    private void handleDeviceAllState() {

    }

    //--处理在线记录数据
    private void handleDeviceOnline() {

    }

    //--处理注册表数据
    private void handleDeviceRegister() {

    }

    /**
     * @Method : countAct
     * @Description : 计算act数值
     * @Params : [deviceCode, nowAct0]
     * @Return : java.lang.Integer
     * @Author : ZhuoNing
     * @Date : 2022/1/18
     * @Time : 15:25
     */
    @Autowired
    private EartagData2Mapper eartagData2Mapper;

    @Override
    public Integer countAct(String earmark, String nowAct,String farmId) {

        EartagData eartagData = eartagData2Mapper.selectOne(new QueryWrapper<EartagData>().eq("farm_id", farmId).eq("earmark", earmark).orderByDesc("ask_time").last("limit 1"));
        Integer lastAct = 0;
        if (ObjectUtil.isNotEmpty(eartagData)){
            lastAct = eartagData.getAct();
        }
        int nowActInt = Integer.parseInt(nowAct);
        return  nowActInt < lastAct ?  nowActInt: nowActInt - lastAct;


        //读取hash
//        Object actObj = redisTemplate.opsForHash().get(deviceCode, "act");
//        if (actObj == null) {
//            //--初始化最新的redis记录
//            initRedisObj(deviceCode);
//            Object lastAct = redisTemplate.opsForHash().get(deviceCode, "act");
//            if (lastAct == null) {
//                log.error("#--- redis数据库有问题，请检查redis是否能正常连接 ---# ");
//                return 0;
//            }
//        }
//        Object lastAct = redisTemplate.opsForHash().get(deviceCode, "act");
//        Integer act1 = Integer.parseInt(nowAct) - Integer.parseInt(lastAct.toString());
////        redisTemplate.opsForHash().put(deviceCode, "act1", act1);
//        if(act1<0){
//            return Integer.parseInt(nowAct);
//        }
//        return act1;
    }

    /**
     * 初始化redis最新记录
     */
    private void initRedisObj(String deviceCode) {
        EartagDataEntity lastEartagDataEntity = eartagDataRepo.getLastEartagData(deviceCode);
        if (lastEartagDataEntity != null) {
            System.out.println("## lastEartagDataEntity.toString>>" + lastEartagDataEntity.toString());
            //--从数据库读取最新记录并覆盖redis
            Map<String, String> map = new HashMap<>();
            map.put("cmdHeader", lastEartagDataEntity.getCmdHeader());
            map.put("device", lastEartagDataEntity.getDevice());
            map.put("earmark", lastEartagDataEntity.getEarmark());
            map.put("bat", lastEartagDataEntity.getBat().toString());
            map.put("earTemp", lastEartagDataEntity.getEarTemp().toString());
            map.put("earTemp1", lastEartagDataEntity.getEarTemp1().toString());
            map.put("envTemp", lastEartagDataEntity.getEnvTemp().toString());
            map.put("envTemp1", lastEartagDataEntity.getEnvTemp1().toString());
            map.put("act", lastEartagDataEntity.getAct().toString());
            map.put("act1", lastEartagDataEntity.getAct1().toString());
            map.put("signal1", lastEartagDataEntity.getSignal1().toString());
            map.put("askTime", lastEartagDataEntity.getAskTime());
            map.put("other", lastEartagDataEntity.getOther());
            map.put("addTime", lastEartagDataEntity.getAddTime().toString());
            //为hash结构设置多个键值对(hmset)
            redisTemplate.opsForHash().putAll(deviceCode, map);
        }
    }
}
